#############################################################################
##
#W    smithform.gd   share package 'homology'      Dumas
##
##==========================================================
## Homology Groups in GAP 4, Smith form routines
## (c) July 1999
##==========================================================

#==================================================
# Sparse Matrix format
#==================================================
# Number of rows, number of columns, character 'M'
# row index, column index, value
# in lexicographic order, starting at 1,
# ended by 0 0 0.
#


#==================================================
# Conversions GAP dense matrix & sparse matrix
#==================================================
#############################################################################
##
#F  gap2SM(<matrix>, <filename>)
##  
##  The `<matrix>' is written to a file named by string `<filename>'.
##  `<filename>' is created or overwritten.
##  A sparse matrix format is used which consists of a series of triples.
##  The first triple is number of rows, number of columns, and the letter ``M''.
##  The last triple is ``0 0 0'' used as an end marker.
##  The intervening triples signify non-zero entries and have the form
##  row-index, col-index, value.  
##  These triples must be in increasing lexicographical order.  
##  Row column pairs must be distinct. 
##  All values must be machine integers.
##
##  For example :
## 
##  32 3200 M
##  1 1 345
##  1 2 -23
##  2 1 77
##  2 200 31
##  30 3101 11
##  0 0 0
##  
##  denotes a 32 by 3200 matrix with 5 non-zero entries, three of which
##  are clustered at the top left.
DeclareGlobalFunction ("gap2SM");

#==================================================
# Computing information about sparse matrices
#==================================================
#############################################################################
##
#F  SMInvariantFactors(<matrix or file>:HomologyAlgorithm := <alg>);
##  
##  returns a list of the form `[ r, c, [f_1,n_1],[f_2, n_2], ... [f_k,n_k]]',
##  where `<r>' and `<c>' are the numbers of rows and columns of the matrix 
##  respectively, and the $f_i$ are the distinct non-zero invariant factors 
##  (Smith form diagonal entries) and $n_i$ are the multiplicity of the $f_i$.
##  Thus the sum of the $n_i$ is the rank of the matrix.
DeclareGlobalFunction ("SMInvariantFactors");

#############################################################################
##
#F  SMSmithForm(<matrix or file>:HomologyAlgorithm := <alg>);
##  
##  returns the same information as SMInvariantFactors, but in expanded form.
##  The format is `[ r, c, [s_1 .. s_{min(r,c)}]]', where again `<r>' and 
##  `<c>' denote the number of rows and columns.  The third item is the 
##  diagonal of the Smith form (it includes the zero entries).
DeclareGlobalFunction ("SMSmithForm");

#############################################################################
##
#F  SMIntegerRank(<matrix or file>:HomologyAlgorithm := <alg>);
##  
##  returns the rank of the matrix over $Z$.  For large matrices,
##  HomologyAlgorithm := ValenceElimAlgorithm is likely to be fastest and be 
##  significantly faster than computing the entire Smith form and determining 
##  the rank from the result.
DeclareGlobalFunction ("SMIntegerRank");

#############################################################################
##
#F  SMPrimeRank(<matrix or file>, <p>:HomologyAlgorithm := <alg>);
##  
##  returns the rank of the matrix mod `<p>', i.e. over $Z/pZ$.  
##  For large matrices, HomologyAlgorithm := ValenceElimAlgorithm is likely 
##  to be fastest and be significantly faster than computing the entire 
##  Smith form and determining the rank from the result.
DeclareGlobalFunction ("SMPrimeRank");

#############################################################################
##
#F  SMPrimePowerRank(<matrix or file>, <n>:HomologyAlgorithm := <alg>);
##  
##  returns the ``rank'' of the matrix mod `<n>', for `<n>' a power of a 
##  prime.  This is not the McCoy rank (largest $i$ such that the 
##  determinantal divisor $d_i$ is nonzero mod $n$), 
##  but is the largest index $i$ such that the $i$-th invariant factor 
##  (smith form entry), $s_i = d_i / d_{i-1}$, is non-zero.
##  For large matrices, HomologyAlgorithm := ValenceElimAlgorithm
##  is likely to be fastest.  
##  
##  ValenceElimAlgorithm and ValenceBBAlgorithm compute integer rank and 
##  prime rank in the same way. Currently ValenceBBAlgorithm cannot compute
##  prime power rank
DeclareGlobalFunction ("SMPrimePowerRank");
